﻿using MyPhotoAlbum.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Items Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234233

namespace MyPhotoAlbum
{
    /// <summary>
    /// A page that displays a collection of item previews.  In the Split Application this page
    /// is used to display and select one of the available groups.
    /// </summary>
    public sealed partial class FolderPage : Page
    {
        private NavigationHelper navigationHelper;
        private ObservableDictionary defaultViewModel = new ObservableDictionary();
        private string currentId;

        /// <summary>
        /// This can be changed to a strongly typed view model.
        /// </summary>
        public ObservableDictionary DefaultViewModel
        {
            get { return this.defaultViewModel; }
        }

        /// <summary>
        /// NavigationHelper is used on each page to aid in navigation and 
        /// process lifetime management
        /// </summary>
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        public FolderPage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);
            this.navigationHelper.LoadState += navigationHelper_LoadState;
        }

        /// <summary>
        /// Populates the page with content passed during navigation.  Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="sender">
        /// The source of the event; typically <see cref="NavigationHelper"/>
        /// </param>
        /// <param name="e">Event data that provides both the navigation parameter passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested and
        /// a dictionary of state preserved by this page during an earlier
        /// session.  The state will be null the first time a page is visited.</param>
        private async void navigationHelper_LoadState(object sender, LoadStateEventArgs e)
        {
            string id = "";
            if (e.NavigationParameter != null && e.NavigationParameter.ToString() != "")
            {
                dynamic obj = e.NavigationParameter;
                id = obj.id;
            }
            currentId = id;
            this.DefaultViewModel["Items"] = await App.DataProvider.GetFolderItems(id, "folders,albums");
        }

        #region NavigationHelper registration

        /// The methods provided in this section are simply used to allow
        /// NavigationHelper to respond to the page's navigation methods.
        /// 
        /// Page specific logic should be placed in event handlers for the  
        /// <see cref="GridCS.Common.NavigationHelper.LoadState"/>
        /// and <see cref="GridCS.Common.NavigationHelper.SaveState"/>.
        /// The navigation parameter is available in the LoadState method 
        /// in addition to page state preserved during an earlier session.

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            navigationHelper.OnNavigatedTo(e);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            navigationHelper.OnNavigatedFrom(e);
        }

        #endregion

        private void itemGridView_ItemClick(object sender, ItemClickEventArgs e)
        {
            this.Frame.Navigate(typeof(FolderPage), e.ClickedItem);
        }

        private void ViewAlbum_Click(object sender, RoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(AlbumPage), currentId);
        }

        private async void CreateFolder_Click(object sender, RoutedEventArgs e)
        {
            if (!String.IsNullOrEmpty(txtFolder.Text))
            {
                if (await App.DataProvider.CreateFolder(currentId, txtFolder.Text))
                {
                    MessageDialog diag = new MessageDialog("Folder was created");
                    diag.ShowAsync();
                    this.DefaultViewModel["Items"] = await App.DataProvider.GetFolderItems(currentId, "folders,albums");

                }
                else
                {
                    MessageDialog diag = new MessageDialog("Folder could not be created");
                    diag.ShowAsync();

                }
            }
            else
            {
                MessageDialog diag = new MessageDialog("Must enter a folder name");
                diag.ShowAsync();
            }
        }

        private void DeleteFolder_Click(object sender, RoutedEventArgs e)
        {
            MessageDialog diag = new MessageDialog("Delete this folder?");

            // Add commands and set their callbacks; both buttons use the same callback function instead of inline event handlers
            diag.Commands.Add(new UICommand(
                "Create",
                new UICommandInvokedHandler(this.DeleteFolderHandler)));
            diag.Commands.Add(new UICommand(
                "Cancel",
                new UICommandInvokedHandler(this.DeleteFolderHandler)));
            diag.ShowAsync();
        }
        private async void DeleteFolderHandler(IUICommand command)
        {
            if (command.Label == "Create")
            {
                if (await App.DataProvider.DeleteFolder(currentId))
                {
                    MessageDialog diag = new MessageDialog("Folder was deleted");
                    await diag.ShowAsync();
                    if(navigationHelper.CanGoBack()) {
                        navigationHelper.GoBack();
                    }
                }
                else
                {
                    MessageDialog diag = new MessageDialog("Folder could not be deleted");
                    diag.ShowAsync();
                }
            }
        }
    }
}
